using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Diagnostics;
using NUnit.Framework;

namespace Circles
{

	/// <summary>
	/// First cut at WindowMaker DockApp-like things, a .NET programming exercise
	/// </summary>
	/// Nelson Minar nelson@monkey.org 2002-09-22
	public class Circles : System.Windows.Forms.Form
	{
		private System.ComponentModel.Container components = null;
		private Graphics screenGraphics, offScreenGraphics;
		private Timer timer;
		private int n = 0;
		private System.Windows.Forms.Panel panel1;
		private Bitmap offScreen;
		private Circle [] circles;
		private System.Windows.Forms.ContextMenu panelMenu;
		private System.Windows.Forms.MenuItem closeItem;
		private System.Windows.Forms.MenuItem aboutItem;
		private int maxSize;
		private Rectangle screenSize;

		public Circles()
		{
			InitializeComponent();

			// Drawing structures for double buffering
			screenGraphics = Graphics.FromHwnd(panel1.Handle);
			offScreen = new Bitmap(panel1.Width, panel1.Height);
			offScreenGraphics = Graphics.FromImage(offScreen);

			// Generate a bunch of random circles
			maxSize = Math.Max(panel1.Width, panel1.Height) * 2 / 3;
			circles = new Circle[10];
			Random r = new Random();
			for (int i = 0; i < circles.Length; i++) 
			{
				float hue = (float)(r.NextDouble()*0.333 + 0.5);
				Pen pen = new Pen(colorFromAhsb(1.0f, hue, 0f, 0.9f));
				pen.Width = 1.5f;
				Brush brush = new SolidBrush(colorFromAhsb(0.2f, hue, 0.37f, 0.8f));

				circles[i] = new Circle(r.Next(panel1.Width), r.Next(panel1.Height), 
					r.Next(maxSize),
					pen, brush);
			}

			// Figure out the screen size
			Screen s = Screen.PrimaryScreen;
			screenSize = s.Bounds;

			// Mouse event handler
			panel1.MouseDown += new MouseEventHandler(this.panel1_MouseDown);
			panel1.MouseUp += new MouseEventHandler(this.panel1_MouseUp);
			panel1.MouseMove += new MouseEventHandler(this.panel1_MouseMove);

			// Set up a timer for our animation loop
			timer = new Timer(); 
			timer.Interval = 10;
			timer.Enabled = true;
			timer.Tick += new EventHandler(OnTimer);
			timer.Start();
		}

		/// <summary>
		/// Handle timer events - main driver of animation.
		/// </summary>
		/// <param name="source"></param>
		/// <param name="e"></param>
		protected void OnTimer(object source, EventArgs e)
		{ 
			offScreenGraphics.Clear(panel1.BackColor);
			for (int i = 0; i < circles.Length; i++) 
			{
				int size = (n + circles[i].N) % (maxSize*2);
				if (size > maxSize)
					size = maxSize*2 - size;
				offScreenGraphics.DrawEllipse(circles[i].Pen, circles[i].X - size/2, circles[i].Y - size/2, size, size);
				offScreenGraphics.FillEllipse(circles[i].Brush, circles[i].X - size/2, circles[i].Y - size/2, size, size);
			}
			screenGraphics.DrawImage(offScreen, 0, 0); 

			n++;
		}

		/// <summary>
		/// Construct a Color object from the given alpha, hue, saturation, and brightness.
		/// These HSB values aren't the same as those returned from Color.getHue(), etc.
		/// </summary>
		/// <param name="alpha">Alpha value in the range [0, 1.0]</param>
		/// <param name="h">Hue in the range [0, 1.0). Only fractional portion is used</param>
		/// <param name="s">Saturation in the range [0, 1.0]</param>
		/// <param name="b">Brightness in the range [0, 1.0]</param>
		/// <returns>Color object</returns>
		public static Color colorFromAhsb(float alpha, float h, float s, float b) 
		{
			// Algorithm from http://mini.net/tcl/666
			float red, green, blue;
			if (s == 0) 
			{					// pure grey
				red = b;
				green = b;
				blue = b;
			} 
			else 
			{
				h = (h - (float)Math.Floor(h));   // Restrict hue to [0, 1)
				float h6 = h * 6.0f;

				// Interpret hue
				red   = (h6 <= 3) ? (2 - h6) : (h6 - 4);
				green = (h6 <= 2) ? (h6) : ((h6 <= 5) ? (4 - h6) : (h6 - 6));
				blue  = (h6 <= 1) ? (-h6) : ((h6 <= 4) ? (h6 - 2) : (6 - h6));

				// Normalize RGB to [0, 1]
				red = (red < 0) ? 0 : ((red > 1) ? 1 : red);
				green = (green < 0) ? 0 : ((green > 1) ? 1 : green);
				blue = (blue < 0) ? 0 : ((blue > 1) ? 1 : blue);

				// Factor in saturation and brightness
				red = ((red - 1) * s + 1) * b;
				green = ((green - 1) * s + 1) * b;
				blue = ((blue - 1) * s + 1) * b;
			}
			return Color.FromArgb((int)(alpha*255), (int)(red*255), (int)(green*255), (int)(blue*255));
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.panel1 = new System.Windows.Forms.Panel();
			this.panelMenu = new System.Windows.Forms.ContextMenu();
			this.aboutItem = new System.Windows.Forms.MenuItem();
			this.closeItem = new System.Windows.Forms.MenuItem();
			this.SuspendLayout();
			// 
			// panel1
			// 
			this.panel1.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.panel1.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(63)), ((System.Byte)(72)), ((System.Byte)(99)));
			this.panel1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.panel1.ContextMenu = this.panelMenu;
			this.panel1.Location = new System.Drawing.Point(1, 1);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(62, 62);
			this.panel1.TabIndex = 0;
			// 
			// panelMenu
			// 
			this.panelMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.aboutItem,
																					  this.closeItem});
			// 
			// aboutItem
			// 
			this.aboutItem.Index = 0;
			this.aboutItem.Text = "About";
			this.aboutItem.Click += new System.EventHandler(this.aboutItem_Click);
			// 
			// closeItem
			// 
			this.closeItem.Index = 1;
			this.closeItem.Text = "Close";
			this.closeItem.Click += new System.EventHandler(this.closeItem_Click);
			// 
			// Circles
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(64, 64);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.panel1});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
			this.MinimumSize = new System.Drawing.Size(1, 1);
			this.Name = "Circles";
			this.Text = "Circles2";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Circles());
		}

		private Point moveOrigin;
		private bool moving = false;
		private void panel1_MouseDown(object sender, MouseEventArgs e) 
		{
			if (e.Button == MouseButtons.Right)
				panel1.ContextMenu.Show(this, new Point(e.X, e.Y));
			else 
			{
				moveOrigin = new Point(e.X, e.Y);
				moving = true;
			}
		}


		private void panel1_MouseMove(object sender, MouseEventArgs e) 
		{
			if (!moving)
				return;
			this.Location = calculateNewLocation(e);
		}

		/// <summary>
		/// Snap the location of the window if it gets near the edge.
		/// </summary>
		/// <param name="e"></param>
		/// <returns></returns>
		private Point calculateNewLocation(MouseEventArgs e) {
			int newX = this.Location.X + e.X - moveOrigin.X;
			int newY = this.Location.Y + e.Y - moveOrigin.Y;

			if (newX > screenSize.Width-(this.Width*3/2)) 
			{
				newX = screenSize.Width-this.Width;
				newY = (newY / this.Width) * this.Width;
			} 
			else if (newX < screenSize.X + (this.Width/2)) 
			{
				newX = 0;
				newY = (newY / this.Width) * this.Width;
			}
			return new Point(newX, newY);
		}

		private void panel1_MouseUp(object sender, MouseEventArgs e) 
		{
			this.Location = calculateNewLocation(e);
			moving = false;
		}

		private void aboutItem_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show(this, "Circles2, by Nelson Minar", "About");
		}

		private void closeItem_Click(object sender, System.EventArgs e)
		{
			Application.ExitThread();
		}
	}


	/// <summary>
	/// Simple data structure class holding details for each circle.
	/// </summary>
	public struct Circle 
	{
		public Circle(int X, int Y, int N, Pen pen, Brush brush) 
		{
			this.X = X;
			this.Y = Y;
			this.N = N;
			this.Pen = pen;
			this.Brush = brush;
		}
		public int X;
		public int Y;
		public int N;
		public Pen Pen;
		public Brush Brush;
	}


	/// <summary>
	/// Test code for colorFromAhsb(). Requires NUnit 2.0 RC3 or later.
	/// </summary>
	[TestFixture]
	public class ColorTest
	{
		[Test]
		public void Hue() 
		{
			assertRGB(255, 0, 0, Circles.colorFromAhsb(1.0f, 0f, 1f, 1f));
			assertRGB(0, 255, 0, Circles.colorFromAhsb(1.0f, 0.33333f, 1f, 1f));
			assertRGB(0, 0, 255, Circles.colorFromAhsb(1.0f, 0.66666f, 1f, 1f));
		}

		[Test]
		public void Saturation() 
		{
			assertRGB(0, 255, 0, Circles.colorFromAhsb(1.0f, 0.33333f, 1f, 1f));
			assertRGB(127, 255, 127, Circles.colorFromAhsb(1.0f, 0.33333f, 0.5f, 1f));
			assertRGB(255, 255, 255, Circles.colorFromAhsb(1.0f, 0.33333f, 0f, 1f));
		}

		[Test]
		public void Value() 
		{
			assertRGB(0, 255, 0, Circles.colorFromAhsb(1.0f, 0.33333f, 1f, 1f));
			assertRGB(0, 127, 0, Circles.colorFromAhsb(1.0f, 0.33333f, 1.0f, 0.5f));
			assertRGB(0, 0, 0, Circles.colorFromAhsb(1.0f, 0.33333f, 1f, 0f));
		}

		[Test]
		[Ignore (".NET doesn't match up with my code.")]
		public void FormsValues() 
		{
			Color c = Circles.colorFromAhsb(1.0f, 0.4f, 0.7f, 0.2f);
			Assertion.AssertEquals(0.7f, c.GetSaturation());
			Assertion.AssertEquals(0.2f, c.GetBrightness());
			Assertion.AssertEquals(0.4f*360, c.GetHue());
		}

		[Test]
		public void Grey() 
		{
			assertRGB(255, 255, 255, Circles.colorFromAhsb(1.0f, 1.0f, 0.0f, 1.0f));
			assertRGB(0, 0, 0, Circles.colorFromAhsb(1.0f, 1.0f, 0.0f, 0.0f));
			assertRGB(127, 127, 127, Circles.colorFromAhsb(1.0f, 1.0f, 0.0f, 0.5f));
		}

		private void assertRGB(int r, int g, int b, Color c) 
		{
			if (!(c.R == r && c.G == g && c.B == b))
				Assertion.Fail("Expected " + r + "," + g + "," + b + " but was " + c.R + "," + c.G + "," + c.B);
		}
	}

}
